@@ -1,12 +1,12 @@
 #include "moar.h"
 
 /* This representation's function pointer table. */
-static const MVMREPROps this_repr;
+static const MVMREPROps MVMContext_this_repr;
 
 /* Creates a new type object of this representation, and associates it with
  * the given HOW. Also sets the invocation protocol handler in the STable. */
 static MVMObject * type_object_for(MVMThreadContext *tc, MVMObject *HOW) {
-    MVMSTable *st = MVM_gc_allocate_stable(tc, &this_repr, HOW);
+    MVMSTable *st = MVM_gc_allocate_stable(tc, &MVMContext_this_repr, HOW);
 
     MVMROOT(tc, st, {
         MVMObject *obj = MVM_gc_allocate_type_object(tc, st);
@@ -40,7 +40,6 @@ static void at_key(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *d
             "Lexical with name '%s' does not exist in this frame",
                 c_name);
     }
-    MVM_string_flatten(tc, name);
     MVM_HASH_GET(tc, lexical_names, name, entry);
     if (!entry) {
         char *c_name = MVM_string_utf8_encode_C_string(tc, name);
@@ -76,7 +75,6 @@ static void bind_key(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void
                 c_name);
     }
 
-    MVM_string_flatten(tc, name);
     MVM_HASH_GET(tc, lexical_names, name, entry);
     if (!entry) {
         char *c_name = MVM_string_utf8_encode_C_string(tc, name);
@@ -117,7 +115,6 @@ static MVMint64 exists_key(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
     MVMString *name = (MVMString *)key;
     if (!lexical_names)
         return 0;
-    MVM_string_flatten(tc, name);
     MVM_HASH_GET(tc, lexical_names, name, entry);
     return entry ? 1 : 0;
 }
@@ -161,10 +158,10 @@ static void compose(MVMThreadContext *tc, MVMSTable *st, MVMObject *info) {
 
 /* Initializes the representation. */
 const MVMREPROps * MVMContext_initialize(MVMThreadContext *tc) {
-    return &this_repr;
+    return &MVMContext_this_repr;
 }
 
-static const MVMREPROps this_repr = {
+static const MVMREPROps MVMContext_this_repr = {
     type_object_for,
     MVM_gc_allocate_object,
     NULL, /* initialize */
