@@ -1,12 +1,18 @@
 #include "moar.h"
 
 /* This representation's function pointer table. */
-static const MVMREPROps this_repr;
+static const MVMREPROps MVMHash_this_repr;
+
+MVM_STATIC_INLINE MVMString * get_string_key(MVMThreadContext *tc, MVMObject *key) {
+    if (!key || REPR(key)->ID != MVM_REPR_ID_MVMString || !IS_CONCRETE(key))
+        MVM_exception_throw_adhoc(tc, "MVMHash representation requires MVMString keys");
+    return (MVMString *)key;
+}
 
 /* Creates a new type object of this representation, and associates it with
  * the given HOW. */
 static MVMObject * type_object_for(MVMThreadContext *tc, MVMObject *HOW) {
-    MVMSTable *st = MVM_gc_allocate_stable(tc, &this_repr, HOW);
+    MVMSTable *st = MVM_gc_allocate_stable(tc, &MVMHash_this_repr, HOW);
 
     MVMROOT(tc, st, {
         MVMObject *obj = MVM_gc_allocate_type_object(tc, st);
@@ -17,39 +23,32 @@ static MVMObject * type_object_for(MVMThreadContext *tc, MVMObject *HOW) {
     return st->WHAT;
 }
 
-MVM_STATIC_INLINE void extract_key(MVMThreadContext *tc, void **kdata, size_t *klen, MVMObject *key) {
-    MVM_HASH_EXTRACT_KEY(tc, kdata, klen, key, "MVMHash representation requires MVMString keys")
-}
-
 /* Copies the body of one object to another. */
 static void copy_to(MVMThreadContext *tc, MVMSTable *st, void *src, MVMObject *dest_root, void *dest) {
     MVMHashBody *src_body  = (MVMHashBody *)src;
     MVMHashBody *dest_body = (MVMHashBody *)dest;
-    MVMHashEntry *current, *tmp;
+    MVMHashEntry *current  = NULL, *tmp = NULL;
     unsigned bucket_tmp;
 
     /* NOTE: if we really wanted to, we could avoid rehashing... */
     HASH_ITER(hash_handle, src_body->hash_head, current, tmp, bucket_tmp) {
-        size_t klen;
-        void *kdata;
         MVMHashEntry *new_entry = MVM_fixed_size_alloc(tc, tc->instance->fsa,
             sizeof(MVMHashEntry));
-        MVM_ASSIGN_REF(tc, &(dest_root->header), new_entry->key, current->key);
+        MVMString *key = MVM_HASH_KEY(current);
         MVM_ASSIGN_REF(tc, &(dest_root->header), new_entry->value, current->value);
-        extract_key(tc, &kdata, &klen, new_entry->key);
-
-        HASH_ADD_KEYPTR(hash_handle, dest_body->hash_head, kdata, klen, new_entry);
+        MVM_HASH_BIND(tc, dest_body->hash_head, key, new_entry);
+        MVM_gc_write_barrier(tc, &(dest_root->header), &(key->common.header));
     }
 }
 
 /* Adds held objects to the GC worklist. */
 static void gc_mark(MVMThreadContext *tc, MVMSTable *st, void *data, MVMGCWorklist *worklist) {
-    MVMHashBody *body = (MVMHashBody *)data;
-    MVMHashEntry *current, *tmp;
+    MVMHashBody     *body = (MVMHashBody *)data;
+    MVMHashEntry *current = NULL, *tmp = NULL;
     unsigned bucket_tmp;
 
     HASH_ITER(hash_handle, body->hash_head, current, tmp, bucket_tmp) {
-        MVM_gc_worklist_add(tc, worklist, &current->key);
+        MVM_gc_worklist_add(tc, worklist, &current->hash_handle.key);
         MVM_gc_worklist_add(tc, worklist, &current->value);
     }
 }
@@ -57,7 +56,7 @@ static void gc_mark(MVMThreadContext *tc, MVMSTable *st, void *data, MVMGCWorkli
 /* Called by the VM in order to free memory associated with this object. */
 static void gc_free(MVMThreadContext *tc, MVMObject *obj) {
     MVMHash *h = (MVMHash *)obj;
-    MVMHashEntry *current, *tmp;
+    MVMHashEntry *current = NULL, *tmp = NULL;
     unsigned bucket_tmp;
     HASH_ITER(hash_handle, h->body.hash_head, current, tmp, bucket_tmp) {
         if (current != h->body.hash_head)
@@ -69,13 +68,10 @@ static void gc_free(MVMThreadContext *tc, MVMObject *obj) {
         MVM_fixed_size_free(tc, tc->instance->fsa, sizeof(MVMHashEntry), tmp);
 }
 
-static void at_key(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMObject *key, MVMRegister *result, MVMuint16 kind) {
-    MVMHashBody *body = (MVMHashBody *)data;
-    void *kdata;
-    MVMHashEntry *entry;
-    size_t klen;
-    extract_key(tc, &kdata, &klen, key);
-    HASH_FIND_CACHE(hash_handle, body->hash_head, kdata, klen, ((MVMString *)key)->body.cached_hash_code, entry);
+static void at_key(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMObject *key_obj, MVMRegister *result, MVMuint16 kind) {
+    MVMHashBody   *body = (MVMHashBody *)data;
+    MVMHashEntry *entry = NULL;
+    MVM_HASH_GET(tc, body->hash_head, get_string_key(tc, key_obj), entry);
     if (kind == MVM_reg_obj)
         result->o = entry != NULL ? entry->value : tc->instance->VMNull;
     else
@@ -83,30 +79,26 @@ static void at_key(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *d
             "MVMHash representation does not support native type storage");
 }
 
-static void bind_key(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMObject *key, MVMRegister value, MVMuint16 kind) {
-    MVMHashBody *body = (MVMHashBody *)data;
-    void *kdata;
-    MVMHashEntry *entry;
-    size_t klen;
+static void bind_key(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMObject *key_obj, MVMRegister value, MVMuint16 kind) {
+    MVMHashBody   *body = (MVMHashBody *)data;
+    MVMHashEntry *entry = NULL;
 
-    extract_key(tc, &kdata, &klen, key);
+    MVMString *key = get_string_key(tc, key_obj);
+    if (kind != MVM_reg_obj)
+        MVM_exception_throw_adhoc(tc,
+            "MVMHash representation does not support native type storage");
 
     /* first check whether we can must update the old entry. */
-    HASH_FIND_CACHE(hash_handle, body->hash_head, kdata, klen, ((MVMString *)key)->body.cached_hash_code, entry);
+    MVM_HASH_GET(tc, body->hash_head, key, entry);
     if (!entry) {
         entry = MVM_fixed_size_alloc(tc, tc->instance->fsa,
             sizeof(MVMHashEntry));
-        HASH_ADD_KEYPTR(hash_handle, body->hash_head, kdata, klen, entry);
-    }
-    else
-        entry->hash_handle.key = (void *)kdata;
-    MVM_ASSIGN_REF(tc, &(root->header), entry->key, key);
-    if (kind == MVM_reg_obj) {
         MVM_ASSIGN_REF(tc, &(root->header), entry->value, value.o);
+        MVM_HASH_BIND(tc, body->hash_head, key, entry);
+        MVM_gc_write_barrier(tc, &(root->header), &(key->common.header));
     }
     else {
-        MVM_exception_throw_adhoc(tc,
-            "MVMHash representation does not support native type storage");
+        MVM_ASSIGN_REF(tc, &(root->header), entry->value, value.o);
     }
 }
 
@@ -115,25 +107,18 @@ static MVMuint64 elems(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, voi
     return HASH_CNT(hash_handle, body->hash_head);
 }
 
-static MVMint64 exists_key(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMObject *key) {
-    MVMHashBody *body = (MVMHashBody *)data;
-    void *kdata;
-    MVMHashEntry *entry;
-    size_t klen;
-    extract_key(tc, &kdata, &klen, key);
-
-    HASH_FIND_CACHE(hash_handle, body->hash_head, kdata, klen, ((MVMString *)key)->body.cached_hash_code, entry);
+static MVMint64 exists_key(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMObject *key_obj) {
+    MVMHashBody   *body = (MVMHashBody *)data;
+    MVMHashEntry *entry = NULL;
+    MVM_HASH_GET(tc, body->hash_head, get_string_key(tc, key_obj), entry);
     return entry != NULL;
 }
 
-static void delete_key(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMObject *key) {
+static void delete_key(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMObject *key_obj) {
     MVMHashBody *body = (MVMHashBody *)data;
-    MVMHashEntry *old_entry;
-    size_t klen;
-    void *kdata;
-    extract_key(tc, &kdata, &klen, key);
-
-    HASH_FIND_CACHE(hash_handle, body->hash_head, kdata, klen, ((MVMString *)key)->body.cached_hash_code, old_entry);
+    MVMString *key = get_string_key(tc, key_obj);
+    MVMHashEntry *old_entry = NULL;
+    MVM_HASH_GET(tc, body->hash_head, key, old_entry);
     if (old_entry) {
         HASH_DELETE(hash_handle, body->hash_head, old_entry);
         MVM_fixed_size_free(tc, tc->instance->fsa,
@@ -171,6 +156,34 @@ static void compose(MVMThreadContext *tc, MVMSTable *st, MVMObject *info) {
     /* XXX key and value types will be communicated here */
 }
 
+/* Deserialize the representation. */
+static void deserialize(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMSerializationReader *reader) {
+    MVMHashBody *body = (MVMHashBody *)data;
+    MVMint64 elems = MVM_serialization_read_int(tc, reader);
+    MVMint64 i;
+    for (i = 0; i < elems; i++) {
+        MVMString *key = MVM_serialization_read_str(tc, reader);
+        MVMObject *value = MVM_serialization_read_ref(tc, reader);
+        MVMHashEntry *entry = MVM_fixed_size_alloc(tc, tc->instance->fsa,
+            sizeof(MVMHashEntry));
+        MVM_ASSIGN_REF(tc, &(root->header), entry->value, value);
+        MVM_HASH_BIND(tc, body->hash_head, key, entry);
+    }
+}
+
+/* Serialize the representation. */
+static void serialize(MVMThreadContext *tc, MVMSTable *st, void *data, MVMSerializationWriter *writer) {
+    MVMHashBody *body = (MVMHashBody *)data;
+    MVMHashEntry *current = NULL, *tmp = NULL;
+    unsigned bucket_tmp;
+    MVM_serialization_write_int(tc, writer, HASH_CNT(hash_handle, body->hash_head));
+    HASH_ITER(hash_handle, body->hash_head, current, tmp, bucket_tmp) {
+        MVMString *key = MVM_HASH_KEY(current);
+        MVM_serialization_write_str(tc, writer, key);
+        MVM_serialization_write_ref(tc, writer, current->value);
+    }
+}
+
 /* Set the size of the STable. */
 static void deserialize_stable_size(MVMThreadContext *tc, MVMSTable *st, MVMSerializationReader *reader) {
     st->size = sizeof(MVMHash);
@@ -203,10 +216,10 @@ static MVMuint64 unmanaged_size(MVMThreadContext *tc, MVMSTable *st, void *data)
 
 /* Initializes the representation. */
 const MVMREPROps * MVMHash_initialize(MVMThreadContext *tc) {
-    return &this_repr;
+    return &MVMHash_this_repr;
 }
 
-static const MVMREPROps this_repr = {
+static const MVMREPROps MVMHash_this_repr = {
     type_object_for,
     MVM_gc_allocate_object,
     NULL, /* initialize */
@@ -224,8 +237,8 @@ static const MVMREPROps this_repr = {
     elems,
     get_storage_spec,
     NULL, /* change_type */
-    NULL, /* serialize */
-    NULL, /* deserialize */
+    serialize,
+    deserialize,
     NULL, /* serialize_repr_data */
     NULL, /* deserialize_repr_data */
     deserialize_stable_size,
